# How To Install

The PeppTalk Database wrapper is designed to run as a webservice. 
You can use your favorite way of hosting Python WSGI files to host it.
Here we describe the way to deploy it so that it is configured to work with the example applications we have provided.
These instructions are designed for a Debian based box (like Raspbian or Ubuntu) 

## Installing Dependencies

The PeppTalk Database wrapper requires several dependencies.

### Application Dependencies
If you are not using apt-get as your package manager, install these dependencies with your package manager of choice.

Run:
sudo apt-get install apache2
sudo apt-get install python-dev
sudo apt-get install libapache2-mod-wsgi
sudo apt-get install cython
sudo apt-get install python-pip
sudo pip install bottle
sudo pip install UnQLite 
sudo apt-get install curl

Note that the capitalization matters for UnQLite

## Configuring Apache

We will make minimal changes to apache from its default configuration. 

1. Check if apache is running by typing 

```bash
> curl localhost:80
Running this command should display the following if it is the default page
<html><body><h1>It works!</h1>
<p>This is the default web page for this server.</p>
<p>The web server software is running but no content has been added, yet.</p>
</body></html>
```
You might see some other type of default page.

2. Navigate to the apache home directory. 
It should by default be `cd /etc/apache2/`
This folder has a bunch of conf files and other things. We're going to change `ports.conf` and the provided default site. 
If you run your own server, you will probably want to set up the page as a seperate site.  

you can use the test editor of your choice to edit ports.config such as nano
make sure to edit this file using sudo because the file is read only
3. Edit ports.conf
Find the Lines that says
```
NameVirtualHost *:80
Listen 80
```
and replace 80 with 43508. You can use any port you wish, as long as you keep the other componets of PeppTalk up to date.
if NameVirtualHost does not exist add it in with port 43508

4. Navigate into etc/apache2/sites-enabled

use you prefered text editor to edit the default site. 
It is probably called 000-default
Again make sure to edit this file with sudo
In that file, within the virtual host tags, add the below into the file

``` 
    WSGIScriptAlias /PeppTalk /var/www/pepptalk/app.wsgi

    <Directory /var/www/pepptalk>
        WSGIProcessGroup test
        WSGIApplicationGroup %{GLOBAL}
        Order deny,allow
        Allow from all
    </Directory>
```
Also change the tag `<VirtualHost *:80>` at the top of the file by replacing 80 with 43508

This will serve the pepptalk app WSGI to the `/PeppTalk/` extension of your server URL.

5. Return to the main apache2 directory. 

Open up apache.config in a text editor preceeded by sudo

Near the bottom of apache2.conf, above `Include sites-enabled/`
add the following line to the file:
```
WSGIDaemonProcess test
```

Once apache is configured to serve the WSGI, we need to place the python package and create the WSGI file to serve it. 
## Placing Python Packages
1. `cd /var/www`
2. `sudo mkdir pepptalk`
3. `sudo chmod 777 pepptalk`
	We need to make sure the apache user is able to access our files.
4.  Copy the `DatabaseWrapper` folder provided in the pepp talk download into /var/www.
note you might have to be a super user to added it to pepptalk so find where the database wrapper is and run
sudo cp -r DatabaseWrapper /var/www/pepptalk
5.  `sudo chmod 777 DatabaseWrapper`
Once the package is in place, we need to create the WSGI file that will interact with Mod_WSGI
## Creating WSGI File
1. create the file `app.wsgi` in the pepptalk folder to contain the following. 
``` 
import sys

sys.path.insert(0,"/var/www/pepptalk/DatabaseWrapper")

from pepp_wsgi import application
```
2. `chmod 777 app.wsgi` 

3. You can now start your server by using 
`sudo service apache2 start`

4. Test: 

Run
```bash
curl -H "Content-Type: application/json" -X POST -d '{"name":"weather"}' localhost:43508/PeppTalk/Seeds/
```

Congratulations, your listening server is now configured!!
