__author__ = 'Colin Versteeg'
__version__ = ".1"
import bottle
import wrapper
import re

DBWrapper = bottle.Bottle()

DBWrapper.pepp_talk_db_wrapper = wrapper.Wrapper("/var/www/pepptalk/pepptalk.db")


@DBWrapper.post('/Seeds/')
@DBWrapper.post('/Seeds')
def post_seed():
    """
    From Proposal 0.8 "Habanero"

    Add a Seed with Name $Name$ to the database.
    Put /PeppTalk/Seeds
    {
    "name": $Name1$
    }

    The JSON object can be replaced with a query string, so Put /PeppTalk/Seeds?name='$Name1$'
    will perform an identical action.
    If the Seed is added to the database, the server will respond with a 201 response


    """
    # TODO: Handle spaces in json name in a sane way.
    json = bottle.request.json
    if json is None:
        bottle.abort(400, "Requires JSON")
    name = json['name']
    if name == "":
        return bottle.HTTPResponse(status=204)
    if not re.match('^[\w-]+$', name):
        bottle.abort(400, "Only Alpha-Numeric Permitted in Seed Names")
    resp = DBWrapper.pepp_talk_db_wrapper.add_seed_to_database(name)
    if resp is True:
        resp = bottle.HTTPResponse(status=201)
        print("Created {0}".format(name))
    else:
        resp = bottle.HTTPResponse(status=200)
        print("{0} exists".format(name))
    url = "/Seeds/{0}".format(name)
    resp.set_header('Location', bottle.urljoin(bottle.request.url, url))
    return resp


@DBWrapper.post('/Seeds/<name>')
def post_seed_data(name):
    """
    Add Data to a Seed
Add Data from the Seed with Name $Name$ to the database.
If $Name$ does not exist, it will create it.
We are adding data $D1$ to Seed $Name$, which was retrieved at time $T1$.
The data, $D1$ is a nested JSON document that the user must know the structure of to create the report generator.
Post /PeppTalk/Seeds/$Name$
{
    "data": $D1$,
    "name": $Name1$,
    "timestamp": $T1$
}

The database will also store a timestamp corresponding to when this data was added to the database, and will return the JSON object passed in.

    """
    json = bottle.request.json
    db_flag = True
    if json is None:
        bottle.abort(400, "Requires JSON")
    if name != json['name']:
        bottle.abort(400, "Bad Input")
    if not DBWrapper.pepp_talk_db_wrapper.database_has_seed(name):
        bottle.redirect("/Seeds", 303)
    try:
        db_flag = DBWrapper.pepp_talk_db_wrapper.add_data_to_database(json)
    except KeyError as e:
        bottle.abort(400, "Bad Input")
    if db_flag is False:
        ##I believe it is okay to give useful error messages, since this application shouldn't be exposed to
        ##the world
        bottle.abort(500, "Server Error: Couldn't Store")
    return json


@DBWrapper.get("/Seeds/")
@DBWrapper.get("/Seeds")
def get_list_of_seeds():
    ret = DBWrapper.pepp_talk_db_wrapper.get_seed_list()
    if ret is None or len(ret) is 0:
        return bottle.HTTPResponse(status=204)
    if len(ret) > 0:
        return {"names": ret}


@DBWrapper.get("/Seeds/<name>")
@DBWrapper.get("/Seeds/<name>/")
def get_seed_data(name):
    if not DBWrapper.pepp_talk_db_wrapper.database_has_seed(name):
        bottle.abort(400, "Seed does not exist")
    ret = DBWrapper.pepp_talk_db_wrapper.get_seed_data(name)
    if ret is None:
        return bottle.HTTPResponse(status=204)
    else:
        return ret


@DBWrapper.get("/Seeds/<name>/<id:int>/")
@DBWrapper.get("/Seeds/<name>/<id:int>")
def get_old_seed(name, id):
    if not DBWrapper.pepp_talk_db_wrapper.database_has_seed(name):
        bottle.abort(400, "Seed does not exist")
    ret = DBWrapper.pepp_talk_db_wrapper.get_seed_data_old(name, id)
    if ret is None:
        return bottle.HTTPResponse(status=204)
    else:
        return ret
