import os

__author__ = 'Colin Versteeg'

import unittest
from bottle import HTTPResponse, request, post, tob, HTTPError
from io import BytesIO
import mock
import wrapper

class TestWrapper(unittest.TestCase):
    def setUp(self):
        self.testDB = wrapper.Wrapper("test.db")

    def tearDown(self):
        self.testDB = None
        try:
            os.remove("test.db")
        except:
            pass

    def test_add_seed(self):
        name = "test"
        self.testDB.add_seed_to_database(name)
        self.assertTrue(self.testDB.database_has_seed(name))
        self.assertFalse(self.testDB.add_seed_to_database(name))
        self.assertFalse(self.testDB.add_seed_to_database(name))
        self.assertTrue(self.testDB.add_seed_to_database(name+"_two"))



    def test_add_seeds(self):
        names = ["test","weather","finance"]
        for name in names:
            self.testDB.add_seed_to_database(name)
            self.assertTrue(self.testDB.database_has_seed(name))
        list = self.testDB.get_seed_list()
        self.assertEquals(list.sort(),names.sort())

    def test_store_seed_data(self):
        name = "test"
        self.testDB.add_seed_to_database(name)
        self.assertTrue(self.testDB.database_has_seed(name))
        data = {"name":"test","data":{"hi":84,"low":43}}
        self.testDB.add_data_to_database(data)
        ret = self.testDB.get_seed_data("test")
        self.assertEquals(ret,data)
        data = {"name":"test","data":{"hi":82,"low":22}}
        self.testDB.add_data_to_database(data)
        ret = self.testDB.get_seed_data("test")
        self.assertEquals(ret,data)
        self.assertEquals(self.testDB.get_seed_list(),[name])

    def test_retrieve_nonexistant_data(self):
        self.assertIsNone(self.testDB.get_seed_data("test"))
        self.testDB.add_seed_to_database("test")
        self.assertIsNone(self.testDB.get_seed_data("test"))
        self.testDB.add_data_to_database({"name":"test","data":{"hi":84}})
        self.assertIsNotNone(self.testDB.get_seed_data("test"))