import os
import unittest
import mock
from webtest import TestApp
import json
from wrapper import Wrapper

__author__ = 'Colin Versteeg'
##TODO: Functional testing like from  http://bottlepy.org/docs/0.11/recipes.html#functional-testing-bottle-applications


class TestBottleFunctionality(unittest.TestCase):

    class MockWrapper():
        def __init__(self, db_path):
            self.db = {}

        def add_seed_to_database(self,name):
            if name not in self.db:
                self.db[name] = []
                return True
            return False

        def add_data_to_database(self, json_dat):
            r = json_dat
            self.db[r["name"]].append(r)
            return True

        def database_has_seed(self, name):
            return name in self.db

        def get_seed_list(self):
            return self.db.keys()

        def get_seed_data(self, name):
            if len(self.db[name]) is 0:
                return None
            return self.db[name][-1]

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper',new=MockWrapper(""))
    def test_add_seed(self):
        #I import in this scope so that the patching will happen.
        import pepp_wsgi
        app = TestApp(pepp_wsgi.application)
        resp = app.get("/Seeds")
        self.assertEquals(resp.status_int, 204)
        resp = app.post_json("/Seeds",dict(name="weather"))
        self.assertEquals(resp.status_int, 201)
        self.assertEquals(resp.location[-14:],"/Seeds/weather")
        resp = app.post_json("/Seeds",dict(name="weather"))
        self.assertEquals(resp.status_int, 200)
        self.assertEquals(resp.location[-14:],"/Seeds/weather")
        resp = app.get("/Seeds")
        self.assertEquals(resp.status_int, 200)
        self.assertEquals(resp.json['names'],["weather"])
        resp = app.get("/Seeds/")
        self.assertEquals(resp.status_int, 200)
        self.assertEquals(resp.json['names'],["weather"])




    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper',new=MockWrapper(""))
    def test_add_data(self):
        #I import in this scope so that the patching will happen.
        import pepp_wsgi
        app = TestApp(pepp_wsgi.application)
        resp = app.post_json("/Seeds",dict(name="weather"))
        resp = app.post_json("/Seeds/weather",dict(name="weather",data=dict(high=85,low=50)))
        self.assertEquals(resp.status_int,200)
        self.assertEquals(resp.json,dict(name="weather",data=dict(high=85, low=50)))
        resp = app.post_json("/Seeds/finance",dict(name="finance",data=dict(high=85,low=50)))
        self.assertEquals(resp.status_int,303)
        self.assertEquals(resp.location[-6:], "/Seeds")



    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper',new=MockWrapper(""))
    def test_retrieve_data(self):
        #I import in this scope so that the patching will happen.
        import pepp_wsgi
        app = TestApp(pepp_wsgi.application)
        resp = app.post_json("/Seeds",dict(name="weather"))
        resp = app.post_json("/Seeds/weather",dict(name="weather",data=dict(high=85,low=50)))
        resp = app.get("/Seeds/weather")
        self.assertEquals(resp.status_int,200)
        self.assertEquals(resp.json,dict(name="weather",data=dict(high=85, low=50)))
        resp = app.post_json("/Seeds/weather",dict(name="weather",data=dict(high=82,low=45)))
        resp = app.get("/Seeds/weather")
        self.assertEquals(resp.status_int,200)
        self.assertEquals(resp.json,dict(name="weather",data=dict(high=82, low=45)))



    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper',new=MockWrapper(""))
    def test_bad_things(self):
        import pepp_wsgi
        app = TestApp(pepp_wsgi.application)
        resp = app.post_json("/Seeds", dict(name="a horrible thing"),status=400)
        self.assertEquals(resp.html.find("pre").text,"Only Alpha-Numeric Permitted in Seed Names")
        resp = app.post_json("/Seeds",dict(name="a#horrible%thing"),status=400)

        resp = app.get("/Seeds/weather",status=400)

        self.assertEquals(resp.html.find("pre").text,"Seed does not exist")
        app.post_json("/Seeds",dict(name="weather"))

        resp = app.get("/Seeds/weather")
        self.assertEquals(resp.status_int,204)

        resp = app.post_json("/Seeds",dict(name=""))
        self.assertEquals(resp.status_int,204)

        resp = app.post("/Seeds",status=400)
        self.assertEquals(resp.html.find("pre").text,"Requires JSON")

        resp = app.post("/Seeds/weather",status=400)
        self.assertEquals(resp.html.find("pre").text,"Requires JSON")

        resp = app.post_json("/Seeds/weather",dict(name="finance"),status=400)
        self.assertEquals(resp.html.find("pre").text,"Bad Input")

class TestFullFunctionality(unittest.TestCase):

    def tearDown(self):
        print("Test Over")


    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper',new=Wrapper("test.db"))
    def test_bad_things_real(self):
        import pepp_wsgi
        app = TestApp(pepp_wsgi.application)
        resp = app.post_json("/Seeds", dict(name="a horrible thing"),status=400)
        self.assertEquals(resp.html.find("pre").text,"Only Alpha-Numeric Permitted in Seed Names")
        resp = app.post_json("/Seeds",dict(name="a#horrible%thing"),status=400)

        resp = app.get("/Seeds/weather",status=400)

        self.assertEquals(resp.html.find("pre").text,"Seed does not exist")
        app.post_json("/Seeds",dict(name="weather"))

        resp = app.get("/Seeds/weather")
        self.assertEquals(resp.status_int,204)

        resp = app.post_json("/Seeds",dict(name=""))
        self.assertEquals(resp.status_int,204)

        resp = app.post("/Seeds",status=400)
        self.assertEquals(resp.html.find("pre").text,"Requires JSON")

        resp = app.post("/Seeds/weather",status=400)
        self.assertEquals(resp.html.find("pre").text,"Requires JSON")

        resp = app.post_json("/Seeds/weather",dict(name="finance"),status=400)
        self.assertEquals(resp.html.find("pre").text,"Bad Input")


    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper',new=Wrapper("test.db"))
    def test_retrieve_data_forreal(self):
        #I import in this scope so that the patching will happen.
        import pepp_wsgi
        app = TestApp(pepp_wsgi.application)
        resp = app.post_json("/Seeds",dict(name="weather"))
        resp = app.post_json("/Seeds/weather",dict(name="weather",data=dict(high=85,low=50)))
        resp = app.get("/Seeds/weather")
        self.assertEquals(resp.status_int,200)
        self.assertEquals(resp.json,dict(name="weather",data=dict(high=85, low=50)))


    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper',new=Wrapper("test.db"))
    def test_add_seed_real(self):
        #I import in this scope so that the patching will happen.
        import pepp_wsgi
        app = TestApp(pepp_wsgi.application)
        resp = app.get("/Seeds")
        self.assertEquals(resp.status_int, 204)
        resp = app.post_json("/Seeds",dict(name="weather"))
        self.assertEquals(resp.status_int, 201)
        self.assertEquals(resp.location[-14:],"/Seeds/weather")
        resp = app.post_json("/Seeds",dict(name="weather"))
        self.assertEquals(resp.status_int, 200)
        self.assertNotEquals(resp.status_int,201)
        self.assertEquals(resp.location[-14:],"/Seeds/weather")
        resp = app.get("/Seeds")
        self.assertEquals(resp.status_int, 200)
        self.assertEquals(resp.json['names'],["weather"])



    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper',new=Wrapper("test.db"))
    def test_add_data_real(self):
        #I import in this scope so that the patching will happen.
        import pepp_wsgi
        app = TestApp(pepp_wsgi.application)
        resp = app.post_json("/Seeds",dict(name="weather"))
        resp = app.post_json("/Seeds/weather",dict(name="weather",data=dict(high=85,low=50)))
        self.assertEquals(resp.status_int,200)
        self.assertEquals(resp.json,dict(name="weather",data=dict(high=85, low=50)))
        resp = app.post_json("/Seeds/finance",dict(name="finance",data=dict(high=85,low=50)))
        self.assertEquals(resp.status_int,303)
        self.assertEquals(resp.location[-6:], "/Seeds")
