"""
Sample Output for AAPL, VTI, and XOM
VTI is up 0.67% to $106.77. The previous close was $106.06. 
AAPL is down -2.16% to $107.48. The previous close was $109.85. 
XOM is up 0.95% to $85.78. The previous close was $84.97.
"""

from seed import Seed
import requests, json
from datetime import datetime

class Stock:
    def __init__(self, name, company, price, prev, change):
        self.name = name
        self.company = company
        self.price = price
        self.prev = prev
        self.change = change

    def encode(self):
        encoded = {}
        encoded['name'] = self.name
        encoded['ticker'] = self.name
        encoded['company'] = self.company
        encoded['price'] = self.price
        encoded['prev_close'] = self.prev
        encoded['percent_change'] = self.change
        return encoded

    def __str__(self):
        direction = 'up' if self.change > 0 else 'down'
        return '{} is {} {}% to ${}. The previous close was ${}.'.format(self.name, direction, self.change, self.price, self.prev)

    def __repr__(self):
        return self.__str__()

class FinanceSeed():
    dbHost = "http://localhost:43508"
    dbPath = "/PeppTalk/Seeds/"

    def __init__(self):
        self.name = "finance"
        self.stocks = None

    def retrieve_data(self, ticker_list): 
        print self.name
        if not ticker_list:
            print "ERORR: Please input a list of stock tickers."
            return None 

        stocks_str = ','.join(['%22' + ticker + '%22' for ticker in ticker_list])
        baseurl = "https://query.yahooapis.com/v1/public/yql?"
        yql_query = 'q=select * from yahoo.finance.quotes where symbol IN ({})'.format(stocks_str)
        yql_url = baseurl + yql_query + '&format=json&env=http://datatables.org/alltables.env'
        data = requests.get(yql_url).json()

        stocks = {}

        if len(ticker_list) is 1:
            if data['query']['results']['quote']['LastTradePriceOnly'] is None: 
                print "ERROR: Please enter a list of valid stock tickers, you entered {}.".format(ticker_list)
                return None
            else: 
                quote = data['query']['results']['quote']
                change = float(quote['ChangeinPercent'][:-1])
                prev = float(quote['PreviousClose'])
                price = float(quote['LastTradePriceOnly'])
                name = quote['Name']

                stock = Stock(ticker_list[0], name, price, prev, change)
                stocks[ticker_list[0]] = stock
        else:
            for ticker in range(0, len(ticker_list)):
                if data['query']['results']['quote'][ticker]['LastTradePriceOnly'] is None:
                    print "ERROR: Please enter a list of valid stock tickers, you entered {}.".format   (ticker_list)
                    return None
                else:
                    quote = data['query']['results']['quote'][ticker]
                    change = float(quote['ChangeinPercent'][:-1])
                    prev = float(quote['PreviousClose'])
                    price = float(quote['LastTradePriceOnly'])
                    name = quote['Name']
        
                    stock = Stock(ticker_list[ticker], name, price, prev, change)
                    stocks[ticker_list[ticker]] = stock
        self.stocks = stocks

    def format_data_for_storage(self):
        stocks_json = {}
        if self.stocks:
            for ticker, quote in self.stocks.iteritems():
                stocks_json[ticker] = quote.encode()

        return Seed(self.name,  stocks_json)
         
    def send_to_database(self, seed):
        finance_json = json.loads(seed.encode())

        url = self.dbHost + self.dbPath
        response = requests.post(url, json=finance_json)
        if (response.status_code != 200) and (response.status_code != 201):
            raise RuntimeError(
                'Failed attempt to post to database. Response from ' + url + ' was ' + str(response.status_code))

        url = url+self.name
        response = requests.post(url, json=finance_json)
        if (response.status_code != 200) and (response.status_code != 201):
            raise RuntimeError(
                'Failed attempt to post to database. Response from ' + url + ' was ' + str(response.status_code))

        return response

    def __str__(self):
        if self.stocks:
            quotes = []
            for ticker, quote in self.stocks.iteritems():
                quotes.append(str(quote))
            return " ".join(quotes)
    
    def __repr__(self):
        return self.__str__()

def main():
    stocks = ["AAPL", "XOM", "VTSAX"]
    finance = FinanceSeed()
    finance.retrieve_data(stocks)
    seed = finance.format_data_for_storage()
    finance.send_to_database(seed)

if __name__ == '__main__':
    main()
    