import requests, json, base64
from seed import Seed

class open_hab_seed():

    openHab_host = "localhost"
    openHab_port = "8080"

    openHab_username = ""
    openHab_pass = ""

    dbHost = "http://localhost:43508"
    dbPath = "/PeppTalk/Seeds/"

    def __init__(self, lightBulbName):
        self.name = "openHab"
        self.lightbulb_name = lightBulbName

    def retrieve_data(self):
        #retrieves the lightbulb state using a call to the OpenHab rest api
        url = 'http://%s:%s/rest/items/%s'%(self.openHab_host, self.openHab_port, self.lightbulb_name)

        payload = {'type': 'json'}

        response = requests.get(url, params=payload, headers=self.basic_header()).json()
        #if(response.status_code != 200):
        #    raise RuntimeError('Response from ' + url + " had status code " + str(response.status_code))

        state = response['state'] #check to make sure state is the correct field to be checking
        return state

    def format_data_for_storage(self, lightbulb_state):
        #formats the data as a json and then stores it in the db
        return  Seed(self.name, self.encode_bulb(lightbulb_state))

    def send_to_database(self, seed):
        openHabJson = json.loads(seed.encoded)

        url = self.dbHost + self.dbPath
        response = requests.post(url, json=openHabJson)
        if (response.status_code != 200) and (response.status_code != 201):
            raise RuntimeError(
                'Failed attempt to post to database. Response from ' + url + ' was ' + str(response.status_code))

        url = url+self.name
        response = requests.post(url, json=openHabJson)
        if (response.status_code != 200) and (response.status_code != 201):
            raise RuntimeError(
                'Failed attempt to post to database. Response from ' + url + ' was ' + str(response.status_code))

        return response

    def encode_bulb(self, lightbulb_state):
        #stores the openHab data in a dictionary
        encoded = {}
        encoded['lightBulbState'] = lightbulb_state
        encoded['lightBulbName'] = self.lightbulb_name
        return encoded

    def basic_header(self):
        """ Header for OpenHAB REST request - standard """

        self.auth = base64.encodestring('%s:%s'
                                        % (self.openHab_username, self.openHab_pass)
                                        ).replace('\n', '')
        return {
            "Authorization" : "Basic %s" %self.auth,
            "Accept": "application/json"}

def main():
    openHabSeed = open_hab_seed(lightBulbName="insteonBulb") #double check this
    lightbulb_state = openHabSeed.retrieve_data()
    seed = openHabSeed.format_data_for_storage(lightbulb_state)
    openHabSeed.send_to_database(seed)

if __name__ == '__main__':
    main()