import unittest
from open_hab_seed import open_hab_seed
import requests_mock
import json

class TestOpenHabSeed(unittest.TestCase):
    def setUp(self):
        self.openHabSeed = open_hab_seed("testOpenHab")
        self.adapter = requests_mock.Adapter()

    @requests_mock.Mocker()
    def test_retrieve_data_success(self, m):
        m.register_uri('GET','http://localhost:8080/rest/items/testOpenHab?type=json',
                       json={'state':'100'}, status_code=200)
        state = self.openHabSeed.retrieve_data()
        self.assertEqual(state, '100')

    @requests_mock.Mocker()
    def test_format_data_for_storage_success(self, m):
        encoded_json = json.loads(self.openHabSeed.format_data_for_storage('100').encoded)
        encoded_json['timestamp'] = 'current_time'
        m.register_uri('POST', 'http://localhost:43508/PeppTalk/Seeds/', status_code=201)
        m.register_uri('POST', 'http://localhost:43508/PeppTalk/Seeds/openHab', json=encoded_json, status_code=200)

        response = self.openHabSeed.send_to_database(self.openHabSeed.format_data_for_storage('100'))
        self.assertEqual(response.status_code, 200, 'Expected response code was ' + str(200)
                         + '. Given response code was ' + str(response.status_code))
        self.assertEqual(response.json()['data'], self.openHabSeed.encode_bulb('100'))
        self.assertEqual(response.json()['data']['lightBulbName'], self.openHabSeed.lightbulb_name)
        self.assertEqual(response.json()['data']['lightBulbState'], '100')
        self.assertTrue('timestamp' in response.json())

    @requests_mock.Mocker()
    def test_format_data_for_storage_creation_failure(self, m):
        m.register_uri('POST', 'http://localhost:43508/PeppTalk/Seeds/', status_code=500)

        with self.assertRaises(RuntimeError):
            self.openHabSeed.send_to_database(self.openHabSeed.format_data_for_storage('100'))

    @requests_mock.Mocker()
    def test_format_data_for_storage_post_failure(self, m):
        m.register_uri('POST', 'http://localhost:43508/PeppTalk/Seeds/', status_code=200)
        m.register_uri('POST', 'http://localhost:43508/PeppTalk/Seeds/openHab', status_code=500)

        with self.assertRaises(RuntimeError):
            self.openHabSeed.send_to_database(self.openHabSeed.format_data_for_storage('100'))