"""
Sample Output for College Park, MD; 3 day forecast
It is mostly cloudy in College Park, MD with a temperature of 62 degrees. Today has a high of 80 and a low of 57.
The forecast for Wednesday is mostly sunny with a high of 71 and a low of 50, Thursday is mostly cloudy with a
high of 75 and a low of 50, Friday is rain with a high of 77 and a low of 62.
"""

from seed import Seed
import requests, json

class Forecast:
    def __init__(self, location, condition, temp, high, low, daily_forecast):
        self.location = location
        self.condition = condition
        self.temp = temp
        self.high = high
        self.low = low
        self.daily_forecast = daily_forecast

    def encode(self):
        encoded = {}
        encoded['location'] = self.location
        encoded['condition'] = self.condition
        encoded['temp'] = self.temp
        encoded['high'] = self.high
        encoded['low'] = self.low
        encoded['forecast'] = []

        for day in self.daily_forecast:
            encoded['forecast'].append(day.encode())

        return encoded

    def __str__(self):
        current = 'It is {} in {} with a temperature of {} degrees. '.format(self.condition, self.location, self.temp)
        today = 'Today has a high of {} and a low of {}. '.format(self.high, self.low)
        weather = ''

        forecast_str = []
        for f in self.daily_forecast:
            forecast_str.append(str(f))

        if forecast_str:
            weather = 'The forecast for '
            weather += ', '.join(forecast_str)
            weather += "."

        return current + today + weather

    def __repr__(self):
        return self.__str__()


class DailyForecast:
    def __init__(self, day, condition, high, low):
        self.day = day
        self.condition = condition
        self.high = high
        self.low = low

    def encode(self):
        encoded = {}
        encoded['day'] = self.day
        encoded['condition'] = self.condition
        encoded['high'] = self.high
        encoded['low'] = self.low
        return encoded

    def __str__(self):
        return '{} is {} with a high of {} and a low of {}'.format(self.day, self.condition, self.high, self.low)

    def __repr__(self):
        return self.__str__()

class WeatherSeed():
    dbHost = "http://localhost:43508"
    dbPath = "/PeppTalk/Seeds/"
    currents = []

    def __init__(self):
        self.name = "weatherPattern"
        self.forecast = None

    def retrieve_data(self, num):
        print self.name
        if num < 0 or num > 5:
            print "ERROR: Please input the number of days you want between 0 and 5."
            return None

        url = self.dbHost + self.dbPath + 'weather'

        for day in range(0, num):
            request_url = url + '/%s'%(str(day))
            response = requests.get(request_url)

            if(response.status_code == 204):
                break

            if(response.status_code != 200):
                raise RuntimeError('Response from ' + request_url + " had status code " + str(response.status_code))

            self.currents.append(response.json()['data']['temp'])

    def encode_patterns(self):
        encoded = {}
        encoded['minTemp'] = min(self.currents)
        encoded['minTempDay'] = self.currents.index(min(self.currents))
        encoded['maxTemp'] = max(self.currents)
        encoded['maxTempDay'] = self.currents.index(max(self.currents))
        encoded['numDays'] = len(self.currents)
        encoded['range'] = encoded['maxTemp'] - encoded['minTemp']

        return encoded

    def format_data_for_storage(self):
        return Seed(self.name, self.encode_patterns())

    def send_to_database(self, seed):
        weather_pattern_json = json.loads(seed.encode())

        url = self.dbHost + self.dbPath
        response = requests.post(url, json=weather_pattern_json)
        if (response.status_code != 200) and (response.status_code != 201):
            raise RuntimeError(
                'Failed attempt to post to database. Response from ' + url + ' was ' + str(response.    status_code))

        url = url+self.name
        response = requests.post(url, json=weather_pattern_json)
        if (response.status_code != 200) and (response.status_code != 201):
            raise RuntimeError(
                'Failed attempt to post to database. Response from ' + url + ' was ' + str(response.status_code))

            return response

    def __str__(self):
        if self.forecast:
            return str(self.forecast)
        else:
            return ""

    def __repr__(self):
        return self.__str__()

def main():
    location = "College Park, MD"
    weather = WeatherSeed()
    weather.retrieve_data(5)
    seed = weather.format_data_for_storage()
    weather.send_to_database(seed)

if __name__ == '__main__':
    main()
