#!/usr/bin/python

import re
import requests
import sys
import os
from genshi.template import NewTextTemplate

# Note: must edit the host and extension once we're done testing.
host = 'http://localhost:43508'
extension = '/PeppTalk/Seeds/'


def database_get_and_transform(path_to_user_template):
    list_of_seeds = get_list_of_seeds(path_to_user_template)
    data_dict = get_data_from_database(list_of_seeds)
    return data_dict


def get_data_from_database(list_of_seeds):
    data_dict = {}
    for seed in list_of_seeds:
        # Slice the quotes around each seed
        seed = seed[1:-1]
        url = host + extension + seed
        response = requests.get(url)

        # Error handling
        if (response.status_code != 200):
            # There was an error. Report and cancel.
            raise RuntimeError('Response from ' + url + ' had status code ' + str(response.status_code))

        # Insert this response's json data into the data dictionary.
        json_data = response.json()
        data_dict[json_data['name']] = json_data['data']
    return data_dict


def get_list_of_seeds(path_to_user_template):
    with open(path_to_user_template, "r") as user_template:
        list_of_seeds = user_template.readline()
        result = re.search('\[(.*)\]', list_of_seeds)
        # Slice the '[' and ']', then slit on commas.
        list_of_seeds = result.group(0)[1:-1].split(',')
    return [f.strip() for f in list_of_seeds]


def generate_pepp_text(path_to_user_template, data_dict):
    pepp_text_string = make_text(data_dict, path_to_user_template)
    # Write the string to a file.
    pepp_text_location = os.path.splitext(path_to_user_template)[0] + '.txt'
    with open(pepp_text_location, 'w') as pepp_text_file:
        pepp_text_file.write(pepp_text_string)
    print("Pepp Text generated at " + pepp_text_location)


def make_text(data_dict, path_to_user_template):
    with open(path_to_user_template) as user_template:
        template = NewTextTemplate(user_template.read())
        stream = template.generate(**data_dict)
        # NOTE: Slice the first character because it is expected to be a new line char.
        # Genshi will skip the comment on the first line but include the '\n' on that line.
        pepp_text_string = stream.render()[1:]

    return pepp_text_string


def main():
    data_dict = database_get_and_transform(sys.argv[1])
    generate_pepp_text(sys.argv[1], data_dict)


if __name__ == '__main__':
    main()
